(function() {'use strict';

    var ScannerHelper = TwitterScannerHelper;

    var Scanner = function() {
        this.ScannerHelper = new ScannerHelper();
        this.responseHeaderScanSingle = TPScanSingleResponse;
        this.responseHeaderScan = TPScanResponse;
    };

    Scanner.prototype.scan = function(callback) {
        var that = this;

        function returnParseDataToUser(parseData) {
            if (parseData['scanResult_incomplete'] == true) {
                // web content parse partly error
                var responseData = new APIResponseHelper(that.responseHeaderScan, ReturnCodeEnum.WEBRESPONSE_PARSE_ERROR_COULD_USE_PARTLY.code, ReturnCodeEnum.WEBRESPONSE_PARSE_ERROR_COULD_USE_PARTLY.description, parseData['scanResult']).getResponse();
            } else {
                // ok
                var responseData = new APIResponseHelper(that.responseHeaderScan, ReturnCodeEnum.OK.code, ReturnCodeEnum.OK.description, parseData['scanResult']).getResponse();
            }
            PSDebug.log(responseData);
            return callback(responseData);
        }

        function sendWebRequestFailCallback(data) {
            // network error
            var responseData = new APIResponseHelper(that.responseHeaderScan, ReturnCodeEnum.NETWORK_ERROR.code, ReturnCodeEnum.NETWORK_ERROR.description).getResponse();
            PSDebug.error(responseData);
            return callback(responseData);
        }

        SendWebRequest(that.ScannerHelper.getSendData_SettingPage(), function(data) {
            var parseData = that.ScannerHelper.handleResponse_SettingPage(TMExt_$(data));

            if (parseData && parseData.user_id && parseData.scanResult) {
                returnParseDataToUser(parseData);
            } else {
                // user not logged in
                var responseData = new APIResponseHelper(that.responseHeaderScan, ReturnCodeEnum.NOT_LOGGED_IN.code, ReturnCodeEnum.NOT_LOGGED_IN.description).getResponse();
                PSDebug.error(responseData);
                callback(responseData);
            }

        }, sendWebRequestFailCallback);
    };

    Scanner.prototype.getAccountInfo = function (target_url, callback) {
        var that = this;

        function sendWebRequestFailCallback(data)
        {
            console.error("Send web request failed!");
            return callback();
        }

        function sendWebRequestOKCallback(data) {
            return callback(that.ScannerHelper.handleAccountInfo(data));
        }

        var params = {
            'type' : 'GET',
            'data' : null,
            'dataType' : 'html',
            'url' : target_url,
            'headers'  : {'connect-src' :'self',  'default-src': 'self', 'font-src' :'self' , 'form-action' :'self',  'frame-src ':'self', 'img-src' :'self','media-src': 'self' , 'object-src': 'none', 'script-src': 'self' ,'style-src' :'self' }
        };

        SendWebRequest(params, sendWebRequestOKCallback, sendWebRequestFailCallback);
    };

    Scanner.prototype.Website = "TWITTER";
    window.TwitterPScanner = Scanner;
})();
