(function() {'use strict';

    // ================================================
    // SendDataConstructor
    // ================================================
    var ScanResultsTemplate = TwitterScanResultsTemplate;

    var ScannerHelper = function() {
        this.PROTOCOL_DOMAIN = PUtil.checkPage.IsFacebook() ? window.location.protocol + '//' + window.location.host : 'https://twitter.com';
    };
    ScannerHelper.prototype.logHeaderSendData = '[Twitter Scan -> send data]';
    ScannerHelper.prototype.logHeaderHandleResponseData = '[Twitter Scan -> handle response data]';

    ScannerHelper.prototype.getUserID = function(data) {
        function method_1(data) {
            try {
                var item = data.find('.current-user .avatar');
                if (item.length != 0) {
                    return item.eq(0).attr('data-user-id');
                } else {
                    return null;
                }
            } catch (e) {
                return null;
            }
        }
        function method_2(data) {
            try {
                var ret = null;
                data.find('[data-user-id]').each(function(index, item) {
                    var id = TMExt_$(item).attr("data-user-id");
                    if(id != "") {
                        ret = id;
                        return false;
                    }
                });
                return ret;
            } catch (e) {
                return null;
            }
        }
        return method_1(data) || method_2(data);
    };
    
    ScannerHelper.prototype.getUserName = function(data) {
        function method_1(data) {
            return data.find('.DashUserDropdown-userInfo .fullname').eq(0).text();
        }
        return method_1(data);
    };
    
    ScannerHelper.prototype.getUserImageURL = function(data) {
        function method_1(data) {
            return data.find('.current-user .avatar').eq(0).attr('src');
        }
        function method_2(data) {
            return data.find('.js-action-profile-avatar').eq(0).attr('src');
        }
        function method_3(data) {
            return data.find('.avatar[data-user-id]').eq(0).attr('src');
        }
        return method_1(data) || method_2(data) || method_3(data);
    };
    
    ScannerHelper.prototype.getSendData_SettingPage = function() {
        var params = {
            'type' : 'GET',
            'data' : null,
            'dataType' : 'html',
            'url' : null
        };

        params.url = this.PROTOCOL_DOMAIN;
        params.url += '/settings/safety';
        new SendDataLogger(this.logHeaderSendData + ' : ' + 'getSendData_SettingPage', params).getLog();
        return params;
    };

    ScannerHelper.prototype.handleResponse_SettingPage = function(data) {
        // data is jquery object
        new ResponseHandlerLogger(this.logHeaderHandleResponseData + ', ID : ' + 'handleResponse_SettingPage', data).getLog();

        var returnResult = {};
        returnResult['user_id'] = null;
        returnResult['scanResult'] = [];
        returnResult['scanResult_incomplete'] = false;

        /*
         get user_id
         */
        try {
            returnResult['user_id'] = this.getUserID(data);
        } catch (e) {
            PSDebug.error(e);
            returnResult['user_id'] = null;
        }

        /*
         get protect_my_tweet
         */
        try {
            var protect_my_tweet = this.getScanResultSampleByID('11');
            protect_my_tweet['Current'] = data.find('#user_protected')[0].checked ? 1 : 0;
            protect_my_tweet['Title'] = data.find('#user_protected_fieldset label.checkbox').text();
            protect_my_tweet['Desc'] = TMExt_$(data.find('#user_protected_fieldset div.controls p')[0].childNodes[0]).text();

            returnResult['scanResult'].push(protect_my_tweet);
        } catch (e) {
            PSDebug.error(e);
            returnResult['scanResult_incomplete'] = true;
        }

        /*
         get let_others_find_me
         */
        try {
            var let_others_find_me = this.getScanResultSampleByID('12');
            let_others_find_me['Current'] = data.find('#user_discoverable_by_email')[0].checked ? 1 : 0;
            let_others_find_me['Title'] = TMExt_$(data.find('#discoverability_fieldset div.controls label.checkbox')[0]).text().replace(/(^\s*)|(\s*$)/g, '');
            let_others_find_me['Desc'] = TMExt_$(data.find('#discoverability_fieldset div.controls label.checkbox')[0]).text().replace(/(^\s*)|(\s*$)/g, '');


            returnResult['scanResult'].push(let_others_find_me);
        } catch (e) {
            PSDebug.error(e);
            returnResult['scanResult_incomplete'] = true;
        }

        /*
         get tailor_twitter
         */
        try {
            var let_others_find_me = this.getScanResultSampleByID('13');
            let_others_find_me['Current'] = data.find('#user_mobile_discoverable')[0].checked ? 1 : 0;
            let_others_find_me['Title'] = TMExt_$(data.find('#discoverability_fieldset div.controls label.checkbox')[1]).text().replace(/(^\s*)|(\s*$)/g, '');
            let_others_find_me['Desc'] = TMExt_$(data.find('#discoverability_fieldset div.controls label.checkbox')[1]).text().replace(/(^\s*)|(\s*$)/g, '');
            returnResult['scanResult'].push(let_others_find_me);
        } catch (e) {
            PSDebug.error(e);
            returnResult['scanResult_incomplete'] = true;
        }

        /*
         get add_a_location
         */
        try {
            var add_a_location = this.getScanResultSampleByID('14');
            add_a_location['Current'] = data.find('#user_geo_enabled')[0].checked ? 1 : 0;
            add_a_location['Title'] = TMExt_$(data.find('#user_geo_enabled')).parent().text();
            add_a_location['Desc'] = add_a_location['Title'];

            returnResult['scanResult'].push(add_a_location);
        } catch (e) {
            PSDebug.error(e);
            returnResult['scanResult_incomplete'] = true;
        }
        
        /*
         personalized_ads
         */
        try {
            var personalized_ads = this.getScanResultSampleByID('15');        
            personalized_ads['Current'] = TMExt_$('#allow_ads_personalization')[0].checked === true ? 1:0;        
            personalized_ads['Title'] =TMExt_$('#allow_ads_personalization')[0].parentNode.textContent;         
            personalized_ads['Desc'] = personalized_ads['Title'];
            returnResult['scanResult'].push(personalized_ads);
        } catch (e) {
            PSDebug.error(e);
            returnResult['scanResult_incomplete'] = true;
        }
        
        /*
         Photo tagging.
         */
        try {
            var photo_tagging = this.getScanResultSampleByID('16');
            
            var label_input = data.find('#allow_media_tagging_fieldset input');
            photo_tagging['Current'] = (function(){
                for(var index = 0; index < label_input.length; index++){
                    if(label_input.eq(index)[0].checked){
                        return index;
                    }
                }
                return 0;
            })();
            
            photo_tagging['Title'] = data.find('#allow_media_tagging_fieldset legend').text();
            photo_tagging['Desc'] = photo_tagging['Title'];
            
            photo_tagging['PossibleValue'] = {
                0: label_input.eq(0).parent().text(),
                1: label_input.eq(1).parent().text(),
                2: label_input.eq(2).parent().text()
            };

            returnResult['scanResult'].push(photo_tagging);
        } catch (e) {
            PSDebug.error(e);
            returnResult['scanResult_incomplete'] = true;
        }

        return returnResult;
    };

    ScannerHelper.prototype.getAllvalidID = function() {
        var allValidID = [];
        for (var i = 0; i < ScanResultsTemplate.length; i++) {
            allValidID.push(ScanResultsTemplate[i]['ID']);
        }
        return allValidID;
    };
    ScannerHelper.prototype.isValidID = function(id) {
        var allValidID = this.getAllvalidID();
        for (var i = 0; i < allValidID.length; i++) {
            if (id == allValidID[i]) {
                return true;
            }
        }
        return false;
    };

    ScannerHelper.prototype.getScanResultSampleByID = function(id) {
        for (var i = 0; i < ScanResultsTemplate.length; i++) {
            if (ScanResultsTemplate[i]['ID'] == id) {
                return PUtil.cloneObj(ScanResultsTemplate[i]);
            }
        }
    };

    ScannerHelper.prototype.isUserNotLogin = function (data) {
        return TMExt_$(data).find("input[name='session[username_or_email]']").length >= 1;
    };

    ScannerHelper.prototype.handleAccountInfo = function (data) {
        var that = this;
        var objHtml = TMExt_$(data);


        var accountInfo = {};
        var user_id = that.getUserID(objHtml);
        if(!user_id && that.isUserNotLogin(objHtml))
        {
            accountInfo.blogin = false;
            return accountInfo;
        }

        accountInfo['blogin'] = true;
        accountInfo['user_id'] = user_id;
        accountInfo['name'] = that.getUserName(objHtml);
        accountInfo['image_url'] = that.getUserImageURL(objHtml);

        return accountInfo;
    };

    window.TwitterScannerHelper = ScannerHelper;
})();
