function wtp_getBrowserType(){
    if(typeof(browser)!="undefined"){
        return __FIREFOX__;
    }else if(typeof(chrome)!="undefined"){
        return __CHROME__;
    }
    if(typeof(safari) != "undefined"){
        return __SAFARI__;
    }
}

function wtp_getCurrentLocale(){
    var user_language;
    var currentLocale;
    if (window.navigator.languages) {
        user_language = window.navigator.languages[0];
    } else {
        user_language = window.navigator.userLanguage || window.navigator.language;
    }
    if (user_language.substring(0, 2) === "en") {
        currentLocale = "en";
    } else if (user_language === "zh" || user_language === "zh-CN") {
        currentLocale = "zh_CN";
    } else if (user_language === "zh-TW" || user_language === "zh-HK") {
        currentLocale = "zh_TW";
    } else if (user_language.substring(0, 2) === "fr") {
        currentLocale = "fr";
    } else if (user_language.substring(0, 2) === "de") {
        currentLocale = "de";
    } else if (user_language.substring(0, 2) === "ja") {
        currentLocale = "ja";
    } else if (user_language.substring(0, 2) === "es") {
        currentLocale = "es";
    } else {
        currentLocale = __DEFAULT_LOCALES__;    // use English as default
    }

    let i = 0;
    for(i = 0; i<__SUPPORT_LOCALES__.length; ++i){
        if(__SUPPORT_LOCALES__[i] == currentLocale){
            break;
        }
    }
    if(i == __SUPPORT_LOCALES__.length){
        currentLocale = __DEFAULT_LOCALES__;
    }
    return currentLocale;
}

function wtp_TSREscapeURL(strURL) {
    return safari.extension.baseURI + strURL;
}

function wpt_getBlockPageUrl(messageName){
    let getURL = null;
    let getMessage = null;

    let browserType = wtp_getBrowserType();
    if(browserType == __CHROME__){
        getURL = chrome.runtime.getURL;
        getMessage = chrome.i18n.getMessage;
    }
    else if(browserType == __FIREFOX__){
        getURL = browser.runtime.getURL;
        getMessage = browser.i18n.getMessage;
    }
    if(browserType != __SAFARI__){
        return getURL(getMessage(messageName));
    }

    //current browser is safari
    let currentLocale = wtp_getCurrentLocale();
    if(messageName == __URLFWTP_HTM__){
        return wtp_TSREscapeURL('Scripts/WTP/local_block_page/' + currentLocale + '/UrlfWTP.htm');
    }
    else if(messageName == __URLFUD_HTM__){
        return wtp_TSREscapeURL('Scripts/WTP/local_block_page/' + currentLocale + '/UrlfUD.htm');
    }
    else if(messageName == __URLFCAT_HTM__){
        return wtp_TSREscapeURL('Scripts/WTP/local_block_page/' + currentLocale + '/UrlfCat.htm');
    }

    return null;
}

function wpt_getUrl(wtp_level,level){
    let path = null;
    if ( level == __DANGEROUS__        || 
         level == __SUSPICIOUS__       || 
         level == __HIGH_SUSPICIOUS__  ){
        //UrlfWTP.htm:wtp_block_file
        if(wtp_level == __LOW__ && 
            (level == __SUSPICIOUS__ || level==__HIGH_SUSPICIOUS__ ) ){
            return null;
        }
        if(wtp_level==__NORMAL__ && 
            (level == __SUSPICIOUS__)){
            return null;
        }
        path = wpt_getBlockPageUrl(__URLFWTP_HTM__);
    }
    else if (level==__BLOCKED__) {
        //UrlfUD.htm:user_defined_block_file
        path = wpt_getBlockPageUrl(__URLFUD_HTM__);
    }
    else if(level==__PC_BLOCKED__){
        //UrlfCat.htm:category_block_file
        path = wpt_getBlockPageUrl(__URLFCAT_HTM__);
    }
    return path;
}

function wtp_getPUMEvent(protocol,path){
    if(path.indexOf("UrlfWTP") != -1){
        if( protocol=="https:" ){
            return "WTP_TB_HTTPS_BLOCK_C_F1";
        }
        else if( protocol=="http:" ){
            return "WTP_TB_HTTP_BLOCK_C_F1";
        }
    }
    else if(path.indexOf("UrlfCat")!=-1){
        if( protocol=="https:" ){
            return "WF_TB_HTTPS_BLOCK_C_F1";
        }
        else if( protocol=="http:" ){
            return "WF_TB_HTTP_BLOCK_C_F1";
        }
    }else if( path.indexOf("UrlfUD")!=-1 ){
        if( protocol=="https:" ){
            return "UD_TB_HTTPS_BLOCK_C_F1";
        }else if( protocol=="http:" ){
            return "UD_TB_HTTP_BLOCK_C_F1";
        }
    }
}

function wtp_dealRatingResponse(response){
    let level           = response.level;
    let curUrl          = document.location.href;
    let wtp_level       = response.wtp_level;
    let category        = response.category;
    let categoryGroup   = response.categoryGroup;

    if( typeof(level)           !="number" ||
        typeof(wtp_level)       !="number" ||
        typeof(categoryGroup)   !="number" ){
        return null;
    }

    if( level != __DANGEROUS__        &&
        level != __SUSPICIOUS__       &&
        level != __BLOCKED__          && 
        level != __PC_BLOCKED__       &&
        level != __HIGH_SUSPICIOUS__  ){
        return null;
    }

    path = wpt_getUrl(wtp_level,level);
    if(path!=null){
        let pramaArr = [category,curUrl,level,categoryGroup];
        let urlPrama = pramaArr.join("|");
        if(urlPrama!=null){
            path += "?code=" + window.btoa(urlPrama);
        }
    }
    return path;
}

function wtp_currentIsBlockPage(){
    if( location.protocol==="https:" ){
        return false;
    }
    if( location.protocol==="http:" &&
        location.port==="443"       ){
        return false;
    }
    var e1 = document.querySelector("td.title_white");
    var e2 = document.querySelector("span.subtitle");
    var e3 = document.querySelector("td.copyright");
    return ((e1!=null) && (e2!=null) && (e3!=null));
}