(function() {
    var exports = window;
    
    var TAG = 'patternRate';
     
    var PagePattern = function() {
    };

    PagePattern.prototype.scanPage = function() {
        Debug.logDebug(TAG, "==> scanPage");
        
        var searchResults = TSRParse();
        if (!searchResults) {
            Debug.logDebug(TAG, 'searchResults is null');
            return;
        }

        if( self.resultLength==searchResults.length ){
            return ;
        }
        self.resultLength=searchResults.length;

        Debug.logDebug(TAG, 'searchResults.length=' + searchResults.length);
        var urls = [];
        var count4sended = 0;
        for (var iCount = 1; iCount < searchResults.length; iCount++) {
            var sr = searchResults[iCount];
            if (!sr || sr.sended) {
                continue;
            }
            count4sended++;
            sr.sended = true;
            var url = { id: iCount, url: sr.link };
            urls.push(url);
            if (urls.length > 3) {
                var that = this;
                this.rateItems(urls.slice(0), function(item) {
                    try {
                        that.onRateResult(item);
                    } catch (e) {
                        Debug.logError(TAG, 'fail to insert result, item=' + JSON.stringify(item) + ', exception=' + e);
                    }
                });
                urls = [];
            }
        }
        if (urls.length > 0) {
            count4sended++;
            var that = this;
            this.rateItems(urls.slice(0), function(item) {
                try {
                    that.onRateResult(item);
                } catch (e) {
                    Debug.logError(TAG, 'fail to insert result, item=' + JSON.stringify(item) + ', exception=' + e);
                }
            });
        }
        Debug.logDebug(TAG, '<== scanPage, count4sended=' + count4sended);
    };

    PagePattern.prototype.onRateResult = function(item) {
        Debug.logDebug(TAG, '==> onRateResult, item=' + JSON.stringify(item));

        /*
         * If the FlowID is not the same,
         * it means this is the last rating result. Abort.
         */
        if (TSRInsertResultICON) {
	        TSRInsertResultICON(item.id, item.result.level, false);
            Debug.logDebug(TAG, 'objDoc.TSRInsertResultICON ok');
        }
    };

    PagePattern.prototype.rateItems = function(urls, onResult) {
        var message = {
            cmd: 'rateItems',
            params: {
                urls: urls,
                type: 'AutomaticScan'
            }
        };

	    Debug.logDebug(TAG, '==> rateItems_sendMessage, message=' + JSON.stringify(message));
	    var _urls = urls;
	    browser.runtime.sendMessage(message, function(response) {
	        Debug.logDebug(TAG, '-------------------------------');
	        Debug.logDebug(TAG, '<== rateItems_sendMessage, message=' + JSON.stringify(message));
	        Debug.logDebug(TAG, '<== rateItems_sendMessage, response=' + JSON.stringify(response));
	        Debug.logDebug(TAG, '-------------------------------');

	        var results = response.results;
	        if (results.length != urls.length) {
		        for (var i = urls.length - 1; i >= results.length; --i) {
			        results.push({
				        level: 0
			        });
		        }
	        }
	        for (var i = _urls.length - 1; i >= 0; --i) {
		        var item = _urls[i];
		        item.result = results[i];
		        Debug.logDebug(TAG, 'id=' + item.id + ', url=' + item.url + ', level=' + item.result.level);
                onResult(item);
	        }
        });
    };

    PagePattern.prototype.monitorDOMChange = function() {
        var that = this;
        var self = this;
        if ( that.observer!=undefined ){
            console.debug("observer is exists.");
            return ;
        }
        let target = document.querySelector("body");
        var config = {childList: true, subtree: true};
        that.observer = new MutationObserver(function(mutations){
            var allOfLinks = document.querySelectorAll("a");
            if( self.aTagNum!=allOfLinks.length ){
                console.debug(`body has changed, start to scan page,${self.aTagNum},${allOfLinks.length}`);
                self.aTagNum = allOfLinks.length;
                that.scanPage();
            }
        });
        that.observer.observe(target, config);
        console.debug("that.observer start to observe.");
    };

    PagePattern.prototype.onDOMChanged = function() {
        var that = this;
        that.cancelMonitorDOMChange();
        that.timerDelay = setTimeout(function() {
            that.scanAndMonitor();
        }, 500);
    };

    PagePattern.prototype.scanAndMonitor = function() {
        console.debug("start to scan and monitor");
        //this.cancelMonitorDOMChange();
        this.scanPage();
        this.monitorDOMChange();
    };

    PagePattern.prototype.checkDOMChange = function() {
        // var currentANumber = document.getElementsByTagName('A').length;
        // if (currentANumber != this.currentANumber) {
        //     Debug.logDebug(TAG, this.currentANumber + '=>' + currentANumber);
        //     try {
        //         this.onDOMChanged();
        //     } catch (e) {
        //         Debug.logError(TAG, 'exception when call onDOMChanged');
        //     }
        // } else {
        //     var self = this;
        //     this.timerMonitor = setTimeout(function() {
        //         self.checkDOMChange();
        //     }, 1000);
        // }
    };

    PagePattern.prototype.cancelMonitorDOMChange = function() {
        // if (this.timerMonitor) {
        //     clearTimeout(this.timerMonitor);
        //     this.timerMonitor = undefined;
        // }
        // if (this.timerDelay) {
        //     clearTimeout(this.timerDelay);
        //     this.timerDelay = undefined;
        // }
        // TMExt_$('body').unbind('DOMNodeInserted');
    };

    PagePattern.prototype.start = function() {
        this.currentANumber = 0;
        this.scanAndMonitor();
    };

    PagePattern.prototype.unLoad = function() {
        Debug.logDebug(TAG, '==>unload');
        this.cancelMonitorDOMChange();
    };

    window.startRate = function() {
        if (window.pagePattern) {
            Debug.logError(TAG, 'you should only call this one time');
            window.pagePattern.unLoad();
            window.pagePattern = undefined;
        }
        window.pagePattern = new PagePattern();
	    // TMExt_$(window).on('unload', window.pagePattern.unload);

	    window.pagePattern.start();
    };

    exports.PagePattern = PagePattern;

})();

