/*global require: true, exports: true*/
var exports = window;
var Debug = window.log;
var TAG = 'ITISBrowserExtServer';

var TMExt_$ = window.TMExt_$;

(function() {
    var ITISBrowserExtServer = function(setting) {
        this.setting = setting;
    };
    
    TMExt_$.extend(ITISBrowserExtServer, {
        URIForPattern: 'http://127.0.0.1:37848/BrowserExt/pattern',
        URIForSetting:
            'http://127.0.0.1:37848/BrowserExt/setting?type=extension',
        URIForUrl: 'http://127.0.0.1:37848/BrowserExt/rating',
        URIForCommand: 'http://127.0.0.1:37848/BrowserExt/command'
    }); 
    
    ITISBrowserExtServer.prototype.enableAutomaticScan = function() {
        Debug.logDebug(TAG, '==>EnableAutomaticRating');
        this._updateSetting('EnableAutomaticScan',
            !this.setting.EnableAutomaticScan);
    };
    
    ITISBrowserExtServer.prototype.enableManualScanner = function() {
        Debug.logDebug(TAG, '==>EnableManualScanner');
        this._updateSetting('EnableManualScanner',
            !this.setting.EnableManualScanner);
    };
    
    ITISBrowserExtServer.showHelp = function() {
        ITISBrowserExtServer._sendCommand('showToolbarHelp');
    };
    
    ITISBrowserExtServer.showPrivacyScannerHelp = function() {
        ITISBrowserExtServer._sendCommand('showPrivacyScannerHelp');
    };
    
    ITISBrowserExtServer.showPrivacyScannerErrorPage = function(helpId) {
        switch (helpId){
            case 0:
                // something wrong
            ITISBrowserExtServer._sendCommand('showPSErrorForSomethingWentWrong');
                break;
            
            case 1:
            // leave it open
            ITISBrowserExtServer._sendCommand('showPSWarningForLeaveItOpen');
                break;
                
            default: 
                break;
        }
    };
    
    ITISBrowserExtServer.prototype.showToolbarUI = function() {
        ITISBrowserExtServer._sendCommand('showToolbarUI');
    };
    
    ITISBrowserExtServer.prototype.loadURLPattern = function(onLoaded) {
        Debug.logDebug(TAG, 'begin load url pattern');
        var that = this;
        this._loadPattern('url', function(data) {
            var _urlpat = {};
            try {
                _urlpat = TMExt_$.parseJSON(data);
            } catch (e) {
                Debug.logError(TAG, 'urlPattern result is not correct,data=',
                    data);
            }
            // TMExt_$.extend(TMExt.urlpattern, _urlpat);
            that.setting.urlpatternLoaded = true;
            if (typeof onLoaded === 'function') {
                onLoaded({
                    status: 'success',
                    pattern: _urlpat
                });
            }
        }, function() {
            if (typeof onLoaded === 'function') {
                onLoaded({
                    status: 'error'
                });
            }
        });
    };
    
    ITISBrowserExtServer.prototype.loadContentPattern = function(id, callback) {
        this._loadPattern('content', callback, null, {
            id: id
        });
    };
    
    ITISBrowserExtServer.prototype.loadFromLocal = function(url, callback) {
        TMExt_$.ajax({
            url: url,
            dataType: 'text',
            success: function(data, statusText, jqXHR) {
                callback(data);
             }
        });
    };
    
    ITISBrowserExtServer.prototype.rateURL = function(urls, onResult) {
        var that = this;
        Debug.logDebug(TAG, '==> enter rateURL');
        var _headers = { 
            TMExtURLs_Length: urls.length
        };
        TMExt_$.each(urls, function(i, item) {
            var encodedUri;
            try {
                var decodedUri = decodeURI(item.url);
                encodedUri = encodeURI(decodedUri);
            } catch(e) {
                encodedUri = item.url;
            }
            
            _headers['TMEExtURLs_' + i + '_url'] = encodedUri;
            _headers['TMEExtURLs_' + i + '_id'] = item.id;
            _headers['TMEExtURLs_' + i + '_needResolving'] = item.needResolving;
        });
        
        var startTime = new Date().getTime();
        TMExt_$.ajax({
            url: ITISBrowserExtServer.URIForUrl + '?timestamp=' + new Date().getTime(),
            async: true,
            headers: _headers,
            dataType: 'text',
            success: function(data, statusText, jqXHR) {
                that._checkAndUpdateSetting(jqXHR);
                Debug.logDebug(TAG, 'rateUrl success, urls=' + TMExt_$.toJSON(urls) +
                    ', data=' + data + ', time=' + (new Date().getTime() - startTime));
    
                var results = {};
                try {
                    results = TMExt_$.parseJSON(data);
                } catch (e) {
                    Debug.logError(TAG, 'rateurl results is not correct, data=' + data);
                }
                onResult(results);
             },
             error: function(jqXHR, textStatus, errorThrown) {
                Debug.logError(TAG, 'fail to rateItems ,textStatus=' + textStatus + ', errorThrown=' + errorThrown);
                onResult({results: []});
             }
        });
    };

    ITISBrowserExtServer.prototype.rateWTPURL = function(urls, onResult) {
        var that = this;
        Debug.logDebug(TAG, '==> enter rateWTPURL');
        var _headers = {
            TMExtURLs_Length: urls.length
        };
        TMExt_$.each(urls, function(i, item) {
            var encodedUri;
            try {
                var decodedUri = decodeURI(item.url);
                encodedUri = encodeURI(decodedUri);
            } catch(e) {
                encodedUri = item.url;
            }

            _headers['TMEExtURLs_' + i + '_url'] = encodedUri;
            _headers['TMEExtURLs_' + i + '_id'] = item.id;
            _headers['TMEExtURLs_' + i + '_needResolving'] = item.needResolving;
        });

        var startTime = new Date().getTime();
        TMExt_$.ajax({
            url: ITISBrowserExtServer.URIForUrl + '?timestamp=' + new Date().getTime() + "&type=wtp",
            async: true,
            headers: _headers,
            dataType: 'text',
            success: function(data, statusText, jqXHR) {
                that._checkAndUpdateSetting(jqXHR);
                Debug.logDebug(TAG, 'rateWTPUrl success, urls=' + TMExt_$.toJSON(urls) +
                    ', data=' + data + ', time=' + (new Date().getTime() - startTime));

                var results = {};
                try {
                    results = TMExt_$.parseJSON(data);
                } catch (e) {
                    Debug.logError(TAG, 'rateurl results is not correct, data=' + data);
                }
                onResult(results);
            },
            error: function(jqXHR, textStatus, errorThrown) {
                Debug.logError(TAG, 'fail to rateWTPItems ,textStatus=' + textStatus + ', errorThrown=' + errorThrown);
                onResult({results: []});
            }
        });
    };
    
    ITISBrowserExtServer.prototype._checkAndUpdateSetting = function(jqXHR) {
        // WORKAROUND: fix issue that ajaxSuccess is
        // fired after success callback.
        // this cause user need to refresh page twice
        // to make setting synced.
        var strSetting = jqXHR.getResponseHeader('X-iTIS-ToolbarSetting');
        var _setting;
        if (strSetting) {
            Debug.logDebug(TAG, 'got X-iTIS-ToolbarSetting header');
            try {
                _setting = TMExt_$.parseJSON(strSetting);
            } catch (e) {
                Debug.logError(TAG, 'setting format is not correct, setting=' + strSetting);
            }
        }
        var strLicense = jqXHR.getResponseHeader('X-iTIS-LicenseStatus');
        if (strLicense) {
            var expired = (strLicense === 'true' ? false : true);
            if (this.setting.LicenseExpired !== expired) {
                if (!_setting) {
                    _setting = {};
                }
                _setting.LicenseExpired = expired;
            }
        }
        var strToolbarStatus = jqXHR.getResponseHeader('X-iTIS-ToolbarStatus');
        if (strToolbarStatus) {
            var enabled = (strToolbarStatus === 'true' ? true : false);
            if (this.setting.ToolbarEnableStatus !== enabled) {
                if (!_setting) {
                    _setting = {};
                }
                _setting.ToolbarEnableStatus = enabled;
            }
        }
        if (this.setting.serverStopped) {
            if (!_setting) {
                _setting = {};
            }
            _setting.serverStopped = false;
        }
        if (_setting) {
            this.setting.update(_setting);
        }
    };
    
    ITISBrowserExtServer.prototype._loadPattern = function(type, 
        callback, onError, options) {
        var that = this;
        TMExt_$.ajax({
            url: ITISBrowserExtServer.URIForPattern,
            dataType: 'text',
            data: TMExt_$.extend({}, {type: type}, options),
            error: function(jqXHR, textStatus, errorThrown) {
                if (onError) {
                    onError();
                }
            },
            success: function(data, statusText, jqXHR) {
                that._checkAndUpdateSetting(jqXHR);
                var _data = jqXHR.responseText;
                callback(data);
            }
        });
    };
    
    ITISBrowserExtServer.prototype._updateSetting = function(name, value) {
        var that = this;
        ITISBrowserExtServer._sendCommand('updateSetting', {
            name: name,
            value: value
        }, function(data) {
            Debug.logDebug(TAG, '<== updateSetting, data=' + JSON.stringify(data));
            that._loadSetting();
        });
    };
    
    ITISBrowserExtServer.prototype._loadSetting = function(onSuccess, onFail) {
        var that = this;
        TMExt_$.ajax({
            url: ITISBrowserExtServer.URIForSetting,
            dataType: 'json',
            async: false,
            success: function(data, textStatus, jqXHR) {
                Debug.logDebug(TAG, 'load setting sucessfully, data=' + JSON.stringify(data));
                that._checkAndUpdateSetting(jqXHR);
                that.setting.update(data);
                
                if (typeof onSuccess === 'function') {
                    onSuccess(JSON.stringify(data));
                }
            },
            error: function(jqXHR, textStatus, errorThrown) {
                Debug.logError(TAG, 'load setting failed, textStatus=' + textStatus + ', errorThrown=' + errorThrown);
                if (typeof onFail === 'function') {
                    onFail();
                }
            }
        });
    };
    
    ITISBrowserExtServer._sendCommand = function(name, params, callback) {
        if (!params) {
            params = {};
        }
        Debug.logDebug(TAG, '==>_sendCommand, name=' + name + ', params=' + JSON.stringify(params));
        TMExt_$.get(ITISBrowserExtServer.URIForCommand,
            TMExt_$.extend({}, {type: name}, params),
            callback);
    };
    
    ITISBrowserExtServer.getPSPromotionSetting = function(callback) {
        TMExt_$.ajax({
            url: ITISBrowserExtServer.URIForSetting,
            dataType: 'json',
            success: function(data, textStatus, jqXHR) {
                Debug.logDebug('ITISBrowserExtServer', 'load setting sucessfully');
                                
                var licenseStatus;
                var strLicense = jqXHR.getResponseHeader('X-iTIS-LicenseStatus');
                if (strLicense) {
                    licenseStatus = (strLicense === 'true' ? true : false);
                }
                
                var toolbarStatus;
                var strToolbarStatus = jqXHR.getResponseHeader('X-iTIS-ToolbarStatus');
                if (strToolbarStatus) {
                    toolbarStatus = (strToolbarStatus === 'true' ? true : false);
                }
                
                var PSPromotionStatus = licenseStatus && toolbarStatus;
                var PSPromotionSetting = data.Promotion;
                if (!PSPromotionStatus) {
                    // set PSPromotion settings to false if license status or toolbar stauts is off
                    PSPromotionSetting.ShowFaceBookPromotion = false;
                    PSPromotionSetting.ShowTwitterPromotion = false;
                    PSPromotionSetting.ShowGooglePlusPromotion = false;
                    PSPromotionSetting.ShowLinkedInPromotion = false;
                }
                
                if (typeof callback === 'function') {
                    callback(PSPromotionSetting);
                }
            }
        });
    };
    
    ITISBrowserExtServer.setPSPromotionSetting = function(key, value) {        
        Debug.logDebug('ITISBrowserExtServer', '==>setPSPromotionSetting, ' + key + '->' + value);
        TMExt_$.get(ITISBrowserExtServer.URIForCommand, {
            type: 'updateSetting',
            name: key,
            value: value
        });
    };
    
    ITISBrowserExtServer.getProductName = function(locale, callback) {
        Debug.logDebug('ITISBrowserExtServer', '==>getProductName, locale=' + locale);
        TMExt_$.get(ITISBrowserExtServer.URIForCommand, {
            type: 'getProductName',
            locale: locale.replace('_', '-')
        }, function(productName) {
            if (typeof callback === 'function') {
                callback(productName);
            }
        });
    };
    
    exports.ITISBrowserExtServer = ITISBrowserExtServer;
})();
