/*global require: true, exports: true*/
var exports = window;
var TMExt_$ = window.TMExt_$;

var L10NHelper = {
    /**
     * window.navigator.languages returns array of preferred languages
     * Generally contents are displayed in 1st language in array
     * Language may be region-related, e.g. en-US
     */
    getBrowserLanguage: function () {
        var user_language;
        var currentLocale;
        // get browser language
        if (window.navigator.languages) {
            // Chrome and Firefox language array
            user_language = window.navigator.languages[0];
        } else {
            // other browser
            user_language = window.navigator.userLanguage || window.navigator.language;
        }

        if (user_language.substring(0, 2) === "en") {
            currentLocale = "en";
        } else if (user_language === "zh" || user_language === "zh-CN") {
            currentLocale = "zh_CN";
        } else if (user_language === "zh-TW" || user_language === "zh-HK") {
            currentLocale = "zh_TW";
        } else if (user_language.substring(0, 2) === "fr") {
            currentLocale = "fr";
        } else if (user_language.substring(0, 2) === "de") {
            currentLocale = "de";
        } else if (user_language.substring(0, 2) === "ja") {
            currentLocale = "ja";
        } else if (user_language.substring(0, 2) === "es") {
            currentLocale = "es";
        } else {
            currentLocale = "en";    // use English as default
        }
        return currentLocale;
    },

    getProductName: function () {
        return L10NHelper.ProductName;
    },

    getPopupSize: function () {
        var width = parseInt(L10NHelper.FirefoxPopupWidth, 10);
        if (isNaN(width)) {
            width = 255;
        }

        var height = parseInt(L10NHelper.FirefoxPopupHeight, 10);
        if (isNaN(height)) {
            height = 145;
        }

        return {
            width: width,
            height: height
        };
    },

    getPopupMenuWordings: function () {
        return L10NHelper.PopupMenuWordings;
    },

    getTooltipTitleString: function () {
        return L10NHelper.D_TSRToolTipTitleString;
    },

    getRatingLevelTooltipString: function () {
        return L10NHelper.g_oRatingLevelToolTipString;
    },

    loadWordings: function (callback) {

        var currentLocale = L10NHelper.getBrowserLanguage();
        var wordingJsPath = '_locales/' + currentLocale + '/TISProToolbarLocalization.json';

        TMExt_$.getJSON(wordingJsPath, function (data) {
            L10NHelper.D_TSRToolTipTitleString = data.D_TSRToolTipTitleString;
            L10NHelper.g_oRatingLevelToolTipString = data.g_oRatingLevelToolTipString;

            var messagesJsPath = '_locales/' + currentLocale + '/messages.json';
            TMExt_$.getJSON(messagesJsPath, function (data) {
                var popupMenuWordings = {
                    automaticscan: data.automaticscan.message,
                    manualscanner: data.manualscanner.message,
                    privacyscanner: data.privacyscanner.message,
                    setting: data.setting.message,
                    help: data.help.message
                };
                L10NHelper.IconTooltip = data.toolbar_name.message;
                L10NHelper.PopupMenuWordings = popupMenuWordings;
                L10NHelper.ProductName = data.product_name.message;
                if (typeof data.firefox_popup_width !== 'undefined') {
                    L10NHelper.FirefoxPopupWidth = data.firefox_popup_width.message;
                }
                if (typeof data.firefox_popup_height !== 'undefined') {
                    L10NHelper.FirefoxPopupHeight = data.firefox_popup_height.message;
                }

                if (typeof callback === 'function') {
                    callback();
                }
            });
        });
    }
};

exports.L10NHelper = L10NHelper;

